#include "PureNetwork.h"
#include "CNullSocket.h"

namespace pun
{
	namespace socket
	{
		CNullSocket::CNullSocket()
		{
			m_pConnection = 0;
		}

		// Destructor
		CNullSocket::~CNullSocket()
		{
			SAFE_RELEASE(&m_pConnection);
		}

		// Listen server.
		RESULT CNullSocket::listenServer(SckListen sckListen)
		{
			// Already initialized
			if(ASSERT_LOG(m_pConnection, "Socket has already been initialized."))
				return E_FAIL;

			// Create socket connection
			createConnection(&m_pConnection);
			if(ASSERT_LOG(SUCCEEDED(m_pConnection->createSocket(sckListen.protocolType, sckListen.addressType, sckListen.bOverlapped)), "Failed to create listen socket."))
				return E_FAIL;

			// Set-up
			if(ASSERT_LOG(SUCCEEDED(m_pConnection->setUseServer(sckListen)), "Failed to setup listen socket."))
				return E_FAIL;

			return S_OK;
		}

		// Connect server.
		RESULT CNullSocket::connectServer(SckConnect sckConnect)
		{
			// Already initialized
			if(ASSERT_LOG(m_pConnection, "Socket has already been initialized."))
				return E_FAIL;

			// Create socket connection
			createConnection(&m_pConnection);
			if(ASSERT_LOG(SUCCEEDED(m_pConnection->createSocket(sckConnect.protocolType, sckConnect.addressType, sckConnect.bOverlapped)), "Failed to create client socket."))
				return E_FAIL;

			// Set-up
			if(ASSERT_LOG(SUCCEEDED(m_pConnection->setUseClient(sckConnect)), "Failed to setup client socket."))
				return E_FAIL;

			return S_OK;
		}

		// Close socket.
		void CNullSocket::closeSocket()
		{

		}

		// is Connected ?
		bool CNullSocket::isConnected()
		{
			// Not initialized.
			if(!m_pConnection)
				return false;

			// Not server.
			if(m_pConnection->getConnectionType() != ECT_CLIENT)
				return false;

			// Not ready.
			if(m_pConnection->getSocketStep() != ECST_CONNECT)
				return false;

			return true;
		}

		// is Listened ?
		bool CNullSocket::isListened()
		{
			// Not initialized.
			if(!m_pConnection)
				return false;

			// Not client.
			if(m_pConnection->getConnectionType() != ECT_SERVER)
				return false;

			// Not ready.
			if(m_pConnection->getSocketStep() != ECST_COMPLETE)
				return false;

			return true;
		}

		// Send Message
		RESULT CNullSocket::sendMessage(IConnection *pReceiver, core::CDataBuffer **ppDataBuffer, c32 nArraySize)
		{
			// Check local state
			if(ASSERT_LOG(m_pConnection, "Command aborted! : sendMessage()"))
				return E_FAIL;

			// Check connection state
			if(m_pConnection->getSocketStep() != ECST_COMPLETE &&  m_pConnection->getSocketStep() != ECST_CONNECT)
			{
				ASSERT_LOG(false, "Wrong Command in the step. : sendMessage()");
				return E_FAIL;
			}

			switch(m_pConnection->getConnectionType())
			{
			case ECT_CLIENT:
				{
					// Send message to server.
					m_pConnection->sendMessage(ppDataBuffer, nArraySize);
				}
				break;

			case ECT_SERVER:
				{
					if(pReceiver)
					{
						// Send message to one man.
						pReceiver->sendMessage(ppDataBuffer, nArraySize);
					}
					else
					{
						// Send message to all.
						core::list<IConnection *>::iterator iterConnection = m_lstConnection.begin();
						for(;iterConnection != m_lstConnection.end();++iterConnection)
						{
							(*iterConnection)->sendMessage(ppDataBuffer, nArraySize);
						}
					}
				}
			}
			return S_OK;
		}

		// On Event
		void CNullSocket::onEvent(EventMsg eventMsg)
		{
			// Not related.
			if(eventMsg.nMsgType != EEMT_SOCKET)
				return;

			switch(eventMsg.nMsgType)
			{
			case ECMT_NEW_CLIENT:
				onNewClient((IConnection *)eventMsg.nValue[0]);
				break;
			}
		}

		// New Client!
		void CNullSocket::onNewClient(IConnection *pConnection)
		{
			// Add in list
			m_lstConnection.push_back(pConnection);

			// Print log
			io::LogPrinter::printf(io::ELL_INFO, __FILE__, __LINE__, "[Pointer:%p]New Connection!!(%s:%d)",this, pConnection->getAddress(), pConnection->getPort());
		}
	};
};